#include "DfpBrowseFileDelegate.h"

#include "base/DeviceIndex.h"
#include "widgets/FileDialog.h"

#include <QPushButton>

namespace {
static const constexpr char *PROP_INDEX = "index";
static const constexpr char *PROP_PATH = "path";

QString getPath(QPushButton *pb) {
    auto path = pb->property(PROP_PATH).toString();
    return path;
}
void setPath(QPushButton *pb, const QString &path) {
    pb->setProperty(PROP_PATH, path);
}
}

DfpBrowseFileDelegate::DfpBrowseFileDelegate(QObject *parent)
    : QItemDelegate(parent)
{
}

QWidget *DfpBrowseFileDelegate::createEditor(QWidget *parent,
                                             const QStyleOptionViewItem &/* option */,
                                             const QModelIndex &index) const
{
    auto *pb = new QPushButton("Browse...", parent);
    connect(pb, &QPushButton::clicked,
            this, &DfpBrowseFileDelegate::browseFile);

    auto devIndex = index.model()->headerData(index.row(), Qt::Vertical, Qt::UserRole);
    pb->setProperty(PROP_INDEX, devIndex);
    setEditorData(pb, index);

    return pb;
}

void DfpBrowseFileDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    auto pb = qobject_cast<QPushButton*>(editor);
    if(!pb)
        return;
    auto path = index.model()->data(index, Qt::EditRole).toString();
    setPath(pb, path);
}

void DfpBrowseFileDelegate::setModelData(QWidget *editor,
                                         QAbstractItemModel *model,
                                         const QModelIndex &index) const
{
    auto *pb = qobject_cast<QPushButton*>(editor);
    if(!pb)
        return;

    auto path = getPath(pb);
    if(model->data(index, Qt::EditRole).toString() != path)
        model->setData(index, path, Qt::EditRole);
}

void DfpBrowseFileDelegate::updateEditorGeometry(QWidget *editor,
                                                 const QStyleOptionViewItem &option,
                                                 const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}

void DfpBrowseFileDelegate::browseFile()
{
    auto pb = qobject_cast<QPushButton*>(sender());
    if(!pb)
        return;
    auto index = pb->property(PROP_INDEX).value<DeviceIndex>();
    if (index.isNull())
        return;

    auto path = pb->property(PROP_PATH).toString();
    QString fileName = FileDialog::getOpenFileName(pb,
                                                   tr("Open Bit file"),
                                                   path,
                                                   tr("Bit Files (*.bit)"));
    if (!fileName.isEmpty()) {
        setPath(pb, fileName);
        commitData(qobject_cast<QWidget*>(pb));
    }
}
