#ifndef DFPFLASHMODEL_H
#define DFPFLASHMODEL_H

#include "base/DeviceIndex.h"
#include "fsm/FsmState.h"
#include "mvc/DeviceIndexTableModel.h"
#include "lib-common/BaseDeviceAppConfig.h"

struct DfpFlashModelPrivate;
class ConfigKey;

enum class DfpFlashModelColumns : int;

class DfpFlashModel : public DeviceIndexTableModel
{
    Q_OBJECT

public:
    DfpFlashModel(QObject *parent = nullptr);
    ~DfpFlashModel() override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &p = QModelIndex()) const override;

    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role) override;

    DeviceIndex deviceIndexOf(int row) const override;
    int rowOf(const DeviceIndex &sourceIndex) const override;
    QMap<int, bool> defaultColumnState() const override;

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;

public slots:
    void setDevState(const DeviceIndex &index, FsmState fsmSt);
    void setConfig(const ConfigKey &, const BaseDeviceAppConfig &newConfig);

private:
    void sendDataChanged(QVector<int> roles = QVector<int>(),
                         int row = -1, int col = -1);
    QVariant data_DisplayRole(const DeviceIndex &index, DfpFlashModelColumns column) const;
    QVariant data_EditRole(const DeviceIndex &index, DfpFlashModelColumns column) const;
    QVariant data_ToolTip(const DeviceIndex &index, DfpFlashModelColumns column) const;
    QVariant data_BackgroundRole(const DeviceIndex &index, DfpFlashModelColumns column) const;

    QString getImagePath(const DeviceIndex &index) const;

    Q_DISABLE_COPY_MOVE(DfpFlashModel)
    Q_DECLARE_PRIVATE(DfpFlashModel)
    DfpFlashModelPrivate * const d_ptr;
};

#endif // DFPFLASHMODEL_H
