#ifndef DFPMAINWINDOW_H
#define DFPMAINWINDOW_H

#include <optional>

#include <QElapsedTimer>

#include "DfpTask.h"
#include "base/DeviceIndex.h"
// #include "flash-prog/FlashDevTask.h"
#include "lib-common/ReadoutAppMainWindow.h"

class ConfigKey;
class FpgaDataBase;
class DeviceDispatcher;
class DfpFsmMain;
class DiscoverDialog;
class EthFlashDevGlobal;
class QLabel;
class ReadoutAppConfigDb;
struct fw_version_t;
enum class FlashImageType;
enum class FsmState;

namespace Ui {
class DfpMainWindow;
}

class DfpMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT
public:
    explicit DfpMainWindow(QWidget *parent = nullptr);
    ~DfpMainWindow() override;
    DeviceStatusTableViewFancy *getDeviceStatusTableView() override;

private Q_SLOTS:
    void on_actionWrite_Golden_Image_triggered();
    void on_actionGet_record_logs_triggered();
    void on_actionVerify_Golden_image_triggered();
    void on_actionVerify_regular_image_triggered();
    void on_actionErase_Golden_part_triggered();
    void on_actionErase_regular_part_triggered();

    void on_pushButtonWrite_clicked();

    void deviceConnected(const DeviceIndex &index, QString fwStr, const fw_version_t &fwRev);
    void imageWritten(const DeviceIndex &index, FlashImageType imageType, const QString &path);
    void liveMagicFailed(const DeviceIndex &index);
    void onFsmStateChange(const FsmState &state, const FsmState &oldState);
    void devFileUpdated(const DeviceIndex &index, const QString &filePath);

    void on_pushButtonRefreshPaths_clicked();

    void on_actionPrint_FSM_input_triggered() const;

    void on_actionSet_common_bit_file_triggered();
    void on_actionSet_auto_detect_folder_triggered();
    void on_actionSingle_check_mode_toggled(bool checked);

public slots:
    void setConfig(const ConfigKey &configKey, const BaseDeviceAppConfig &newConfig) override;

signals:
    void startDevTask(DFP_TASK task, FlashImageType isGold) const;
    void autoDetectFile(const DeviceIndex &i, const QString &path);
    void printFsmInput() const;
    void allowFsmTransit() const;
    void setSingleCheckMode(bool v) const;

private:
    void addDevice(const DeviceIndex &dd);
    void deleteDevice(const DeviceIndex &);
    void saveConfig();
    void updateWriteButtonState() const;

    void startErase(FlashImageType imageType);
    void setPushButtonEnabled() const;
    void setLabelStatus(QString msg, QString color) const;
    bool alienStateCheck();

    Ui::DfpMainWindow *ui;
    ReadoutAppConfigDb *const configDb;
    DfpFsmMain *const fsm;
    QMap<DeviceIndex, QSharedPointer<EthFlashDevGlobal>> devices;
    DeviceDispatcher *const deviceDispatcher;
    QLabel *const lastStatusLabel;
    FpgaDataBase *const dataBase;

    QElapsedTimer taskTimer;
    std::optional<DfpTask> curTask;
    bool singleCheck = false;
};

#endif // DFPMAINWINDOW_H
