#include "DfpTableView.h"

#include <QSortFilterProxyModel>

#include "mvc/MergeColumnsTableModel.h"

#include "DfpFlashModel.h"
#include "DfpFlashModelColumns.h"
#include "DfpBrowseFileDelegate.h"
#include "DfpProgressDelegate.h"
#include "ModularDeviceStatus/ModularDeviceStatusModel.h"

DfpTableView::DfpTableView(QWidget *parent)
    : DeviceStatusTableViewFancy(parent)
    , dfpModel(new DfpFlashModel(this))
    , browseDelegate(new DfpBrowseFileDelegate(this))
    , progressDelegate(new DfpProgressDelegate(this))
{
    addSourceModel(dfpModel);
    connect(proxyModel, &QAbstractItemModel::modelReset,
            this, &DfpTableView::onModelReset);
}
void DfpTableView::onModelReset()
{
    setItemDelegateForColumn(mergeModel->getColumnFromSource<ModularDeviceStatusModel>(
                                 int(ModularDeviceStatusModel::COL_DSB_FLASH)),
                             progressDelegate);
    setItemDelegateForColumn(mergeModel->getColumnFromSource<DfpFlashModel>(
                                 int(DfpFlashModelColumns::COL_BROWSE_PATH)),
                             browseDelegate);
    addDelegates();
}

void DfpTableView::addDelegates()
{
    const auto &m = model();
    int rows = m->rowCount();
    for(int r=0; r<rows; ++r) {
        {
            int col = int(ModularDeviceStatusModel::COL_DSB_FLASH);
            auto absCol = mergeModel->getColumnFromSource<ModularDeviceStatusModel>(col);
            auto i = m->index(r, absCol);
            if(!isPersistentEditorOpen(i)) {
                openPersistentEditor(i);
            }
        }
        {
            int col = int(DfpFlashModelColumns::COL_BROWSE_PATH);
            auto absCol = mergeModel->getColumnFromSource<DfpFlashModel>(col);
            auto i = m->index(r, absCol);
            if(!isPersistentEditorOpen(i)) {
                openPersistentEditor(i);
            }
        }
    }
}
