#include "MscHistView.h"
#include "MscHistTypeDelegate.h"
#include "MscSpinBoxDelegate.h"

#include <QDebug>
#include <QHeaderView>

#include "MscHistColumns.h"

namespace {
const auto DELEGATE_COLS = {static_cast<int>(MscHistColumns::COL_CE_LUT),
                            static_cast<int>(MscHistColumns::COL_HIST_TYPE),
                            static_cast<int>(MscHistColumns::COL_HIST_LUT),
                            static_cast<int>(MscHistColumns::COL_CH),
                            static_cast<int>(MscHistColumns::COL_SLICETIME)};
}

class MscHistDelegate : public MscSpinBoxDelegate
{
public:
    MscHistDelegate(QObject *parent = nullptr) : MscSpinBoxDelegate(parent) {}

protected:
    SpinBoxParams getParams(int col) const override
    {
        switch(static_cast<MscHistColumns>(col)) {
        case MscHistColumns::COL_CE_LUT:
        case MscHistColumns::COL_HIST_LUT:
        {
            static SpinBoxParams p{.base=16,.groupSeparator=false};
            return p;
        }
        case MscHistColumns::COL_CH:
        {
            static SpinBoxParams p{.max=15};
            return p;
        }
        case MscHistColumns::COL_SLICETIME:
        {
            static SpinBoxParams p{.max=int(std::numeric_limits<quint32>::max()/1e3),
                        .min=1};
            return p;
        }
        default:;
        }
        Q_UNREACHABLE();
    }
};

MscHistView::MscHistView(QWidget *parent)
    : QTreeView(parent)
    , sp_delegate(new MscHistDelegate(this))
    , type_delegate(new MscHistTypeDelegate(this))
{
    header()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

void MscHistView::setModel(QAbstractItemModel *model)
{
    QTreeView::setModel(model);

    for(auto col : DELEGATE_COLS)
        setItemDelegateForColumn(static_cast<int>(col), sp_delegate);
    setItemDelegateForColumn(static_cast<int>(MscHistColumns::COL_HIST_TYPE), type_delegate);

    connect(model, &QAbstractItemModel::modelReset,
            this, &MscHistView::onModelReset);
}

void MscHistView::onModelReset()
{
    addDelegate();
}

void MscHistView::addDelegate(const QModelIndex &parent)
{
    const auto &m = model();
    if(parent.isValid()) {
        for(auto col : DELEGATE_COLS) {
            if(col == static_cast<int>(MscHistColumns::COL_CH) &&
                    !parent.parent().isValid())
                continue;
            if(col == static_cast<int>(MscHistColumns::COL_HIST_TYPE) &&
                m->rowCount(parent) != 0)
                continue;
            const auto &i = parent.siblingAtColumn(col);
            if(!isPersistentEditorOpen(i))
                openPersistentEditor(i);
        }
    }

    int r1 = m->rowCount(parent);
    for(int i=0; i<r1; ++i) {
        auto chIndex = m->index(i, 0, parent);
        addDelegate(chIndex);
    }
}
